@@ -69,9 +69,13 @@ pub trait NodeMethods {
     /// Returns all nodes being application windows.
     fn windows(&self) -> Vec<&s::Node>;
 
+    /// Returns all nodes being containers.
+    fn containers(&self) -> Vec<&s::Node>;
+
     /// Returns all nodes being workspaces.
     fn workspaces(&self) -> Vec<&s::Node>;
 
+    // Returns true if this node is the scratchpad workspace.
     fn is_scratchpad(&self) -> bool;
 }
 
@@ -87,16 +91,19 @@ impl NodeMethods for s::Node {
     }
 
     fn is_container(&self) -> bool {
-        self.node_type == s::NodeType::Workspace
-            || self.node_type == s::NodeType::Con
-                && self.name.is_none()
-                && self.layout != s::NodeLayout::None
+        self.node_type == s::NodeType::Con
+            && self.name.is_none()
+            && self.layout != s::NodeLayout::None
     }
 
     fn windows(&self) -> Vec<&s::Node> {
         self.iter().filter(|n| n.is_window()).collect()
     }
 
+    fn containers(&self) -> Vec<&s::Node> {
+        self.iter().filter(|n| n.is_container()).collect()
+    }
+
     fn workspaces(&self) -> Vec<&s::Node> {
         self.iter()
             .filter(|n| n.node_type == s::NodeType::Workspace)
@@ -113,6 +120,7 @@ impl NodeMethods for s::Node {
 pub struct ExtraProps {
     /// Milliseconds since UNIX epoch.
     pub last_focus_time: u128,
+    pub last_focus_time_for_next_prev_seq: u128,
 }
 
 #[derive(Debug)]
@@ -153,6 +161,35 @@ impl Window<'_> {
     pub fn is_focused(&self) -> bool {
         self.node.focused
     }
+
+    pub fn is_floating(&self) -> bool {
+        self.node.node_type == s::NodeType::FloatingCon
+    }
+
+    pub fn get_parent(&self) -> &s::Node {
+        NodeIter::new(self.workspace)
+            .find(|n| {
+                n.nodes.contains(self.node)
+                    || n.floating_nodes.contains(self.node)
+            })
+            .unwrap_or_else(|| panic!("Window {:?} has no parent node!", self))
+    }
+
+    pub fn is_child_of_tiled_container(&self) -> bool {
+        let layout = &self.get_parent().layout;
+        layout == &s::NodeLayout::SplitH || layout == &s::NodeLayout::SplitV
+    }
+
+    pub fn is_child_of_tabbed_or_stacked_container(&self) -> bool {
+        let layout = &self.get_parent().layout;
+        layout == &s::NodeLayout::Tabbed || layout == &s::NodeLayout::Stacked
+    }
+
+    pub fn last_focus_time_for_next_prev_seq(&self) -> u128 {
+        self.extra_props
+            .as_ref()
+            .map_or(0, |wp| wp.last_focus_time_for_next_prev_seq)
+    }
 }
 
 impl PartialEq for Window<'_> {
@@ -286,7 +323,7 @@ impl<'a> DisplayFormat for Window<'a> {
 fn build_windows<'a>(
     root: &'a s::Node,
     include_scratchpad_windows: bool,
-    extra_props: Option<&HashMap<i64, ExtraProps>>,
+    extra_props: &HashMap<i64, ExtraProps>,
 ) -> Vec<Window<'a>> {
     let mut v = vec![];
     for workspace in root.workspaces() {
@@ -297,7 +334,7 @@ fn build_windows<'a>(
         for n in workspace.windows() {
             v.push(Window {
                 node: n,
-                extra_props: extra_props.and_then(|m| m.get(&n.id).cloned()),
+                extra_props: extra_props.get(&n.id).cloned(),
                 workspace,
             })
         }
@@ -308,13 +345,11 @@ fn build_windows<'a>(
 fn build_workspaces<'a>(
     root: &'a s::Node,
     include_scratchpad: bool,
-    extra_props: Option<&HashMap<i64, ExtraProps>>,
+    extra_props: &HashMap<i64, ExtraProps>,
 ) -> Vec<Workspace<'a>> {
     let mut v = vec![];
     for workspace in root.workspaces() {
-        if workspace.windows().is_empty()
-            || workspace.is_scratchpad() && !include_scratchpad
-        {
+        if workspace.is_scratchpad() && !include_scratchpad {
             continue;
         }
 
@@ -323,19 +358,22 @@ fn build_workspaces<'a>(
             .iter()
             .map(|w| Window {
                 node: w,
-                extra_props: extra_props.and_then(|m| m.get(&w.id).cloned()),
+                extra_props: extra_props.get(&w.id).cloned(),
                 workspace,
             })
             .collect();
-        wins.sort();
+        if !extra_props.is_empty() {
+            wins.sort();
+        }
         v.push(Workspace {
             node: workspace,
-            extra_props: extra_props
-                .and_then(|m| m.get(&workspace.id).cloned()),
+            extra_props: extra_props.get(&workspace.id).cloned(),
             windows: wins,
         })
     }
-    v.sort();
+    if !extra_props.is_empty() {
+        v.sort();
+    }
     v
 }
 
@@ -343,11 +381,10 @@ fn build_workspaces<'a>(
 pub fn get_windows<'a>(
     root: &'a s::Node,
     include_scratchpad_windows: bool,
-    extra_props: Option<&HashMap<i64, ExtraProps>>,
+    extra_props: &HashMap<i64, ExtraProps>,
 ) -> Vec<Window<'a>> {
-    let extra_props_given = extra_props.is_some();
     let mut wins = build_windows(root, include_scratchpad_windows, extra_props);
-    if extra_props_given {
+    if !extra_props.is_empty() {
         wins.sort();
     }
     wins
@@ -357,7 +394,7 @@ pub fn get_windows<'a>(
 pub fn get_workspaces<'a>(
     root: &'a s::Node,
     include_scratchpad: bool,
-    extra_props: Option<&HashMap<i64, ExtraProps>>,
+    extra_props: &HashMap<i64, ExtraProps>,
 ) -> Vec<Workspace<'a>> {
     let mut workspaces =
         build_workspaces(root, include_scratchpad, extra_props);
@@ -365,20 +402,6 @@ pub fn get_workspaces<'a>(
     workspaces
 }
 
-pub fn select_window<'a>(
-    prompt: &str,
-    windows: &'a [Window],
-) -> Option<&'a Window<'a>> {
-    util::select_from_menu(prompt, windows)
-}
-
-pub fn select_workspace<'a>(
-    prompt: &str,
-    workspaces: &'a [Workspace],
-) -> Option<&'a Workspace<'a>> {
-    util::select_from_menu(prompt, workspaces)
-}
-
 pub enum WsOrWin<'a> {
     Ws { ws: &'a Workspace<'a> },
     Win { win: &'a Window<'a> },
@@ -408,15 +431,8 @@ impl WsOrWin<'_> {
     }
 }
 
-pub fn select_workspace_or_window<'a>(
-    prompt: &'a str,
-    ws_or_wins: &'a [WsOrWin<'a>],
-) -> Option<&'a WsOrWin<'a>> {
-    util::select_from_menu(prompt, ws_or_wins)
-}
-
 pub struct Workspace<'a> {
-    node: &'a s::Node,
+    pub node: &'a s::Node,
     extra_props: Option<ExtraProps>,
     pub windows: Vec<Window<'a>>,
 }
@@ -430,9 +446,25 @@ impl Workspace<'_> {
         self.node.id
     }
 
+    pub fn get_windows(&self) -> &Vec<Window> {
+        &self.windows
+    }
+
     pub fn is_scratchpad(&self) -> bool {
         self.node.is_scratchpad()
     }
+
+    pub fn is_focused(&self) -> bool {
+        self.node.focused
+    }
+
+    pub fn is_current(&self) -> bool {
+        is_current_container(self.node)
+    }
+}
+
+pub fn is_current_container(node: &s::Node) -> bool {
+    node.focused || NodeIter::new(node).any(|c| c.focused)
 }
 
 impl PartialEq for Workspace<'_> {
