@@ -79,6 +79,14 @@ impl Config {
             .expect("No format.workspace_format defined.")
     }
 
+    // pub fn get_format_container_format(&self) -> String {
+    //     self.format
+    //         .as_ref()
+    //         .and_then(|f| f.container_format.clone())
+    //         .or_else(|| Format::default().container_format)
+    //         .expect("No format.container_format defined.")
+    // }
+
     pub fn get_format_urgency_start(&self) -> String {
         self.format
             .as_ref()
@@ -147,6 +155,7 @@ pub struct Menu {
 pub struct Format {
     window_format: Option<String>,
     workspace_format: Option<String>,
+    //container_format: Option<String>,
     urgency_start: Option<String>,
     urgency_end: Option<String>,
     html_escape: Option<bool>,
@@ -207,6 +216,11 @@ impl Default for Format {
                  <span alpha=\"20000\">({id})</span>"
                     .to_string(),
             ),
+            // container_format: Some(
+            //     "<i>Container {name}</i>   \
+            //      <span alpha=\"20000\">({id})</span>"
+            //         .to_string(),
+            // ),
             html_escape: Some(true),
             urgency_start: Some(
                 "<span background=\"darkred\" foreground=\"yellow\">"
@@ -258,14 +272,18 @@ impl Default for Config {
 
 fn get_config_file_path() -> Box<Path> {
     let proj_dirs = ProjectDirs::from("", "", "swayr").expect("");
-    let config_dir = proj_dirs.config_dir();
-    if !config_dir.exists() {
+    let user_config_dir = proj_dirs.config_dir();
+    if !user_config_dir.exists() {
+        let sys_config_file = Path::new("/etc/xdg/swayr/config.toml");
+        if sys_config_file.exists() {
+            return sys_config_file.into();
+        }
         DirBuilder::new()
             .recursive(true)
-            .create(config_dir)
+            .create(user_config_dir)
             .unwrap();
     }
-    config_dir.join("config.toml").into_boxed_path()
+    user_config_dir.join("config.toml").into_boxed_path()
 }
 
 pub fn save_config(cfg: Config) {
